@@ -26,6 +26,10 @@ EQUAL_DEPENDENCIES = [
     ),
 ]
 
+EQUIVALENT_DEPENDENCIES = [
+    ("scikit-learn==1.0.1", "scikit_learn==1.0.1"),
+]
+
 DIFFERENT_DEPENDENCIES = [
     ("package_one", "package_two"),
     ("packaging>20.1", "packaging>=20.1"),
@@ -78,6 +82,7 @@ DIFFERENT_DEPENDENCIES = [
         ("git+https://git.example.com/MyProject.git@master", ""),
         ("git+https://git.example.com/MyProject.git@v1.0", ""),
         ("git+https://git.example.com/MyProject.git@refs/pull/123/head", ""),
+        ("gopher:/foo/com", ""),
         (None, "==={ws}arbitrarystring"),
         (None, "({ws}==={ws}arbitrarystring{ws})"),
         (None, "=={ws}1.0"),
@@ -164,6 +169,8 @@ class TestRequirementParsing:
         [
             "file:///absolute/path",
             "file://.",
+            "file:.",
+            "file:/.",
         ],
     )
     def test_file_url(self, url: str) -> None:
@@ -503,25 +510,6 @@ class TestRequirementParsing:
             "                ^"
         )
 
-    @pytest.mark.parametrize(
-        "url",
-        [
-            "gopher:/foo/com",
-            "file:.",
-            "file:/.",
-        ],
-    )
-    def test_error_on_invalid_url(self, url: str) -> None:
-        # GIVEN
-        to_parse = f"name @ {url}"
-
-        # WHEN
-        with pytest.raises(InvalidRequirement) as ctx:
-            Requirement(to_parse)
-
-        # THEN
-        assert "Invalid URL" in ctx.exconly()
-
     def test_error_on_legacy_version_outside_triple_equals(self) -> None:
         # GIVEN
         to_parse = "name==1.0.org1"
@@ -648,13 +636,26 @@ class TestRequirementBehaviour:
 
     @pytest.mark.parametrize("dep1, dep2", EQUAL_DEPENDENCIES)
     def test_equal_reqs_equal_hashes(self, dep1: str, dep2: str) -> None:
-        """Requirement objects created from equivalent strings should be equal."""
+        """Requirement objects created from equal strings should be equal."""
         # GIVEN / WHEN
         req1, req2 = Requirement(dep1), Requirement(dep2)
 
         assert req1 == req2
         assert hash(req1) == hash(req2)
 
+    @pytest.mark.parametrize("dep1, dep2", EQUIVALENT_DEPENDENCIES)
+    def test_equivalent_reqs_equal_hashes_unequal_strings(
+        self, dep1: str, dep2: str
+    ) -> None:
+        """Requirement objects created from equivalent strings should be equal,
+        even though their string representation will not."""
+        # GIVEN / WHEN
+        req1, req2 = Requirement(dep1), Requirement(dep2)
+
+        assert req1 == req2
+        assert hash(req1) == hash(req2)
+        assert str(req1) != str(req2)
+
     @pytest.mark.parametrize("dep1, dep2", DIFFERENT_DEPENDENCIES)
     def test_different_reqs_different_hashes(self, dep1: str, dep2: str) -> None:
         """Requirement objects created from non-equivalent strings should differ."""
