@@ -1,43 +1,40 @@
-# Copyright 2014 Donald Stufft
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-# http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-from __future__ import absolute_import, division, print_function
+# This file is dual licensed under the terms of the Apache License, Version
+# 2.0, and the BSD License. See the LICENSE file in the root of this repository
+# for complete details.
 
 import itertools
 import operator
 
 import pytest
 
-from packaging.specifiers import (
-    InvalidSpecifier, LegacySpecifier, Specifier, SpecifierSet,
-)
-from packaging.version import LegacyVersion, Version, parse
-
-from .test_version import VERSIONS, LEGACY_VERSIONS
+from packaging.specifiers import InvalidSpecifier, Specifier, SpecifierSet
+from packaging.version import Version, parse
 
+from .test_version import VERSIONS
 
 LEGACY_SPECIFIERS = [
-    "==2.1.0.3", "!=2.2.0.5", "<=5", ">=7.9a1", "<1.0.dev1", ">2.0.post1",
+    "==2.1.0.3",
+    "!=2.2.0.5",
+    "<=5",
+    ">=7.9a1",
+    "<1.0.dev1",
+    ">2.0.post1",
 ]
 
 SPECIFIERS = [
-    "~=2.0", "==2.1.*", "==2.1.0.3", "!=2.2.*", "!=2.2.0.5", "<=5", ">=7.9a1",
-    "<1.0.dev1", ">2.0.post1", "===lolwat",
+    "~=2.0",
+    "==2.1.*",
+    "==2.1.0.3",
+    "!=2.2.*",
+    "!=2.2.0.5",
+    "<=5",
+    ">=7.9a1",
+    "<1.0.dev1",
+    ">2.0.post1",
 ]
 
 
 class TestSpecifier:
-
     @pytest.mark.parametrize("specifier", SPECIFIERS)
     def test_specifiers_valid(self, specifier):
         Specifier(specifier)
@@ -47,42 +44,40 @@ class TestSpecifier:
         [
             # Operator-less specifier
             "2.0",
-
             # Invalid operator
             "=>2.0",
-
             # Version-less specifier
             "==",
-
             # Local segment on operators which don't support them
             "~=1.0+5",
             ">=1.0+deadbeef",
             "<=1.0+abc123",
             ">1.0+watwat",
             "<1.0+1.0",
-
             # Prefix matching on operators which don't support them
             "~=1.0.*",
             ">=1.0.*",
             "<=1.0.*",
             ">1.0.*",
             "<1.0.*",
-
             # Combination of local and prefix matching on operators which do
             # support one or the other
             "==1.0.*+5",
             "!=1.0.*+deadbeef",
-
-            # Prefix matching cannot be used inside of a local version
+            # Prefix matching cannot be used with a pre-release, post-release,
+            # dev or local version
+            "==2.0a1.*",
+            "!=2.0a1.*",
+            "==2.0.post1.*",
+            "!=2.0.post1.*",
+            "==2.0.dev1.*",
+            "!=2.0.dev1.*",
             "==1.0+5.*",
             "!=1.0+deadbeef.*",
-
             # Prefix matching must appear at the end
             "==1.0.*.5",
-
             # Compatible operator requires 2 digits in the release operator
             "~=1",
-
             # Cannot use a prefix matching after a .devN version
             "==1.0.dev1.*",
             "!=1.0.dev1.*",
@@ -109,7 +104,6 @@ class TestSpecifier:
             "1.0.DEV1",
             "1.0-DEV",
             "1.0-DEV1",
-
             # Various alpha incarnations
             "1.0a",
             "1.0.a",
@@ -131,7 +125,6 @@ class TestSpecifier:
             "1.0.ALPHA1",
             "1.0-ALPHA",
             "1.0-ALPHA1",
-
             # Various beta incarnations
             "1.0b",
             "1.0.b",
@@ -153,7 +146,6 @@ class TestSpecifier:
             "1.0.BETA1",
             "1.0-BETA",
             "1.0-BETA1",
-
             # Various release candidate incarnations
             "1.0c",
             "1.0.c",
@@ -175,7 +167,6 @@ class TestSpecifier:
             "1.0.RC1",
             "1.0-RC",
             "1.0-RC1",
-
             # Various post release incarnations
             "1.0post",
             "1.0.post",
@@ -191,10 +182,8 @@ class TestSpecifier:
             "1.0-POST",
             "1.0-POST1",
             "1.0-5",
-
             # Local version case insensitivity
             "1.0+AbC"
-
             # Integer Normalization
             "1.01",
             "1.0a05",
@@ -205,7 +194,6 @@ class TestSpecifier:
             "1.1.dev09000",
             "00!1.2",
             "0100!0.0",
-
             # Various other normalizations
             "v1.0",
             "  \r \f \v v1.0\t\n",
@@ -231,7 +219,6 @@ class TestSpecifier:
             (">2.0", ">2.0"),
             (">=2.0", ">=2.0"),
             ("~=2.0", "~=2.0"),
-
             # Spaces should be removed
             ("< 2", "<2"),
         ],
@@ -240,7 +227,7 @@ class TestSpecifier:
         spec = Specifier(specifier)
 
         assert str(spec) == expected
-        assert repr(spec) == "<Specifier({0})>".format(repr(expected))
+        assert repr(spec) == f"<Specifier({expected!r})>"
 
     @pytest.mark.parametrize("specifier", SPECIFIERS)
     def test_specifiers_hash(self, specifier):
@@ -251,45 +238,39 @@ class TestSpecifier:
         itertools.chain(
             *
             # Verify that the equal (==) operator works correctly
-            [
-                [(x, x, operator.eq) for x in SPECIFIERS]
-            ]
+            [[(x, x, operator.eq) for x in SPECIFIERS]]
             +
             # Verify that the not equal (!=) operator works correctly
             [
-                [
-                    (x, y, operator.ne)
-                    for j, y in enumerate(SPECIFIERS)
-                    if i != j
-                ]
+                [(x, y, operator.ne) for j, y in enumerate(SPECIFIERS) if i != j]
                 for i, x in enumerate(SPECIFIERS)
             ]
-        )
+        ),
     )
     def test_comparison_true(self, left, right, op):
         assert op(Specifier(left), Specifier(right))
         assert op(left, Specifier(right))
         assert op(Specifier(left), right)
 
+    @pytest.mark.parametrize(("left", "right"), [("==2.8.0", "==2.8")])
+    def test_comparison_canonicalizes(self, left, right):
+        assert Specifier(left) == Specifier(right)
+        assert left == Specifier(right)
+        assert Specifier(left) == right
+
     @pytest.mark.parametrize(
         ("left", "right", "op"),
         itertools.chain(
             *
             # Verify that the equal (==) operator works correctly
-            [
-                [(x, x, operator.ne) for x in SPECIFIERS]
-            ]
+            [[(x, x, operator.ne) for x in SPECIFIERS]]
             +
             # Verify that the not equal (!=) operator works correctly
             [
-                [
-                    (x, y, operator.eq)
-                    for j, y in enumerate(SPECIFIERS)
-                    if i != j
-                ]
+                [(x, y, operator.eq) for j, y in enumerate(SPECIFIERS) if i != j]
                 for i, x in enumerate(SPECIFIERS)
             ]
-        )
+        ),
     )
     def test_comparison_false(self, left, right, op):
         assert not op(Specifier(left), Specifier(right))
@@ -318,7 +299,6 @@ class TestSpecifier:
                 ("2.0+deadbeef", "==2.0+deadbeef"),
                 ("2.0+deadbeef", "==2.0.0+deadbeef"),
                 ("2.0+deadbeef.0", "==2.0.0+deadbeef.00"),
-
                 # Test the equality operation with a prefix
                 ("2.dev1", "==2.*"),
                 ("2a1", "==2.*"),
@@ -327,13 +307,16 @@ class TestSpecifier:
                 ("2b1.dev1", "==2.*"),
                 ("2c1", "==2.*"),
                 ("2c1.post1.dev1", "==2.*"),
+                ("2c1.post1.dev1", "==2.0.*"),
                 ("2rc1", "==2.*"),
+                ("2rc1", "==2.0.*"),
                 ("2", "==2.*"),
+                ("2", "==2.0.*"),
+                ("2", "==0!2.*"),
+                ("0!2", "==2.*"),
                 ("2.0", "==2.*"),
                 ("2.0.0", "==2.*"),
-                ("2.0.post1", "==2.0.post1.*"),
-                ("2.0.post1.dev1", "==2.0.post1.*"),
-
+                ("2.1+local.version", "==2.1.*"),
                 # Test the in-equality operation
                 ("2.1", "!=2"),
                 ("2.1", "!=2.0"),
@@ -341,11 +324,9 @@ class TestSpecifier:
                 ("2.0.1", "!=2.0"),
                 ("2.0.1", "!=2.0.0"),
                 ("2.0", "!=2.0+deadbeef"),
-
                 # Test the in-equality operation with a prefix
                 ("2.0", "!=3.*"),
                 ("2.1", "!=2.0.*"),
-
                 # Test the greater than equal operation
                 ("2.0", ">=2"),
                 ("2.0", ">=2.0"),
@@ -353,7 +334,6 @@ class TestSpecifier:
                 ("2.0.post1", ">=2"),
                 ("2.0.post1.dev1", ">=2"),
                 ("3", ">=2"),
-
                 # Test the less than equal operation
                 ("2.0", "<=2"),
                 ("2.0", "<=2.0"),
@@ -367,42 +347,40 @@ class TestSpecifier:
                 ("2.0c1.post1.dev1", "<=2"),
                 ("2.0rc1", "<=2"),
                 ("1", "<=2"),
-
                 # Test the greater than operation
                 ("3", ">2"),
                 ("2.1", ">2.0"),
                 ("2.0.1", ">2"),
                 ("2.1.post1", ">2"),
                 ("2.1+local.version", ">2"),
-
                 # Test the less than operation
                 ("1", "<2"),
                 ("2.0", "<2.1"),
                 ("2.0.dev0", "<2.1"),
-
                 # Test the compatibility operation
                 ("1", "~=1.0"),
                 ("1.0.1", "~=1.0"),
                 ("1.1", "~=1.0"),
                 ("1.9999999", "~=1.0"),
-
+                ("1.1", "~=1.0a1"),
+                ("2022.01.01", "~=2022.01.01"),
                 # Test that epochs are handled sanely
                 ("2!1.0", "~=2!1.0"),
                 ("2!1.0", "==2!1.*"),
                 ("2!1.0", "==2!1.0"),
                 ("2!1.0", "!=1.0"),
+                ("2!1.0.0", "==2!1.0.*"),
+                ("2!1.0.0", "==2!1.*"),
                 ("1.0", "!=2!1.0"),
                 ("1.0", "<=2!0.1"),
                 ("2!1.0", ">=2.0"),
                 ("1.0", "<2!0.1"),
                 ("2!1.0", ">2.0"),
-
                 # Test some normalization rules
                 ("2.0.5", ">2.0dev"),
             ]
         ]
-        +
-        [
+        + [
             (v, s, False)
             for v, s in [
                 # Test the equality operation
@@ -410,11 +388,9 @@ class TestSpecifier:
                 ("2.1", "==2.0"),
                 ("2.1", "==2.0.0"),
                 ("2.0", "==2.0+deadbeef"),
-
                 # Test the equality operation with a prefix
                 ("2.0", "==3.*"),
                 ("2.1", "==2.0.*"),
-
                 # Test the in-equality operation
                 ("2.0", "!=2"),
                 ("2.0", "!=2.0"),
@@ -426,7 +402,6 @@ class TestSpecifier:
                 ("2.0+deadbeef", "!=2.0+deadbeef"),
                 ("2.0+deadbeef", "!=2.0.0+deadbeef"),
                 ("2.0+deadbeef.0", "!=2.0.0+deadbeef.00"),
-
                 # Test the in-equality operation with a prefix
                 ("2.dev1", "!=2.*"),
                 ("2a1", "!=2.*"),
@@ -435,13 +410,13 @@ class TestSpecifier:
                 ("2b1.dev1", "!=2.*"),
                 ("2c1", "!=2.*"),
                 ("2c1.post1.dev1", "!=2.*"),
+                ("2c1.post1.dev1", "!=2.0.*"),
                 ("2rc1", "!=2.*"),
+                ("2rc1", "!=2.0.*"),
                 ("2", "!=2.*"),
+                ("2", "!=2.0.*"),
                 ("2.0", "!=2.*"),
                 ("2.0.0", "!=2.*"),
-                ("2.0.post1", "!=2.0.post1.*"),
-                ("2.0.post1.dev1", "!=2.0.post1.*"),
-
                 # Test the greater than equal operation
                 ("2.0.dev1", ">=2"),
                 ("2.0a1", ">=2"),
@@ -452,12 +427,10 @@ class TestSpecifier:
                 ("2.0c1.post1.dev1", ">=2"),
                 ("2.0rc1", ">=2"),
                 ("1", ">=2"),
-
                 # Test the less than equal operation
                 ("2.0.post1", "<=2"),
                 ("2.0.post1.dev1", "<=2"),
                 ("3", "<=2"),
-
                 # Test the greater than operation
                 ("1", ">2"),
                 ("2.0.dev1", ">2"),
@@ -472,7 +445,6 @@ class TestSpecifier:
                 ("2.0.post1", ">2"),
                 ("2.0.post1.dev1", ">2"),
                 ("2.0+local.version", ">2"),
-
                 # Test the less than operation
                 ("2.0.dev1", "<2"),
                 ("2.0a1", "<2"),
@@ -486,12 +458,10 @@ class TestSpecifier:
                 ("2.post1", "<2"),
                 ("2.post1.dev1", "<2"),
                 ("3", "<2"),
-
                 # Test the compatibility operation
                 ("2.0", "~=1.0"),
                 ("1.1.0", "~=1.0.0"),
                 ("1.1.post1", "~=1.0.0"),
-
                 # Test that epochs are handled sanely
                 ("1.0", "~=2!1.0"),
                 ("2!1.0", "~=1.0"),
@@ -526,12 +496,10 @@ class TestSpecifier:
     @pytest.mark.parametrize(
         ("version", "spec", "expected"),
         [
-            # Test identity comparison by itself
-            ("lolwat", "===lolwat", True),
-            ("Lolwat", "===lolwat", True),
-            ("1.0", "===1.0", True),
-            ("nope", "===lolwat", False),
             ("1.0.0", "===1.0", False),
+            ("1.0.dev0", "===1.0", False),
+            # Test identity comparison by itself
+            ("1.0", "===1.0", True),
             ("1.0.dev0", "===1.0.dev0", True),
         ],
     )
@@ -572,7 +540,6 @@ class TestSpecifier:
             (">=1.0", "2.0.dev1", False),
             (">=2.0.dev1", "2.0a1", True),
             ("==2.0.*", "2.0a1.dev1", False),
-            ("==2.0a1.*", "2.0a1.dev1", True),
             ("<=2.0", "1.0.dev1", False),
             ("<=2.0.dev1", "1.0a1", True),
         ],
@@ -600,61 +567,51 @@ class TestSpecifier:
     def test_specifier_filter(self, specifier, prereleases, input, expected):
         spec = Specifier(specifier)
 
-        kwargs = (
-            {"prereleases": prereleases} if prereleases is not None else {}
-        )
+        kwargs = {"prereleases": prereleases} if prereleases is not None else {}
 
         assert list(spec.filter(input, **kwargs)) == expected
 
-    @pytest.mark.xfail
-    def test_specifier_explicit_legacy(self):
-        assert Specifier("==1.0").contains(LegacyVersion("1.0"))
-
     @pytest.mark.parametrize(
-        ('spec', 'op'),
+        ("spec", "op"),
         [
-            ('~=2.0', '~='),
-            ('==2.1.*', '=='),
-            ('==2.1.0.3', '=='),
-            ('!=2.2.*', '!='),
-            ('!=2.2.0.5', '!='),
-            ('<=5', '<='),
-            ('>=7.9a1', '>='),
-            ('<1.0.dev1', '<'),
-            ('>2.0.post1', '>'),
-            ('===lolwat', '==='),
-        ]
+            ("~=2.0", "~="),
+            ("==2.1.*", "=="),
+            ("==2.1.0.3", "=="),
+            ("!=2.2.*", "!="),
+            ("!=2.2.0.5", "!="),
+            ("<=5", "<="),
+            (">=7.9a1", ">="),
+            ("<1.0.dev1", "<"),
+            (">2.0.post1", ">"),
+            # === is an escape hatch in PEP 440
+            ("===lolwat", "==="),
+        ],
     )
     def test_specifier_operator_property(self, spec, op):
         assert Specifier(spec).operator == op
 
     @pytest.mark.parametrize(
-        ('spec', 'version'),
+        ("spec", "version"),
         [
-            ('~=2.0', '2.0'),
-            ('==2.1.*', '2.1.*'),
-            ('==2.1.0.3', '2.1.0.3'),
-            ('!=2.2.*', '2.2.*'),
-            ('!=2.2.0.5', '2.2.0.5'),
-            ('<=5', '5'),
-            ('>=7.9a1', '7.9a1'),
-            ('<1.0.dev1', '1.0.dev1'),
-            ('>2.0.post1', '2.0.post1'),
-            ('===lolwat', 'lolwat'),
-        ]
+            ("~=2.0", "2.0"),
+            ("==2.1.*", "2.1.*"),
+            ("==2.1.0.3", "2.1.0.3"),
+            ("!=2.2.*", "2.2.*"),
+            ("!=2.2.0.5", "2.2.0.5"),
+            ("<=5", "5"),
+            (">=7.9a1", "7.9a1"),
+            ("<1.0.dev1", "1.0.dev1"),
+            (">2.0.post1", "2.0.post1"),
+            # === is an escape hatch in PEP 440
+            ("===lolwat", "lolwat"),
+        ],
     )
     def test_specifier_version_property(self, spec, version):
         assert Specifier(spec).version == version
 
     @pytest.mark.parametrize(
         ("spec", "expected_length"),
-        [
-            ("", 0),
-            ("==2.0", 1),
-            (">=2.0", 1),
-            (">=2.0,<3", 2),
-            (">=2.0,<3,==2.4", 3),
-        ],
+        [("", 0), ("==2.0", 1), (">=2.0", 1), (">=2.0,<3", 2), (">=2.0,<3,==2.4", 3)],
     )
     def test_length(self, spec, expected_length):
         spec = SpecifierSet(spec)
@@ -672,155 +629,18 @@ class TestSpecifier:
     )
     def test_iteration(self, spec, expected_items):
         spec = SpecifierSet(spec)
-        items = set(str(item) for item in spec)
+        items = {str(item) for item in spec}
         assert items == set(expected_items)
 
+    def test_specifier_equal_for_compatible_operator(self):
+        assert Specifier("~=1.18.0") != Specifier("~=1.18")
 
-class TestLegacySpecifier:
-
-    @pytest.mark.parametrize(
-        ("version", "spec", "expected"),
-        [
-            (v, s, True)
-            for v, s in [
-                # Test the equality operation
-                ("2.0", "==2"),
-                ("2.0", "==2.0"),
-                ("2.0", "==2.0.0"),
-
-                # Test the in-equality operation
-                ("2.1", "!=2"),
-                ("2.1", "!=2.0"),
-                ("2.0.1", "!=2"),
-                ("2.0.1", "!=2.0"),
-                ("2.0.1", "!=2.0.0"),
-
-                # Test the greater than equal operation
-                ("2.0", ">=2"),
-                ("2.0", ">=2.0"),
-                ("2.0", ">=2.0.0"),
-                ("2.0.post1", ">=2"),
-                ("2.0.post1.dev1", ">=2"),
-                ("3", ">=2"),
-
-                # Test the less than equal operation
-                ("2.0", "<=2"),
-                ("2.0", "<=2.0"),
-                ("2.0", "<=2.0.0"),
-                ("2.0.dev1", "<=2"),
-                ("2.0a1", "<=2"),
-                ("2.0a1.dev1", "<=2"),
-                ("2.0b1", "<=2"),
-                ("2.0b1.post1", "<=2"),
-                ("2.0c1", "<=2"),
-                ("2.0c1.post1.dev1", "<=2"),
-                ("2.0rc1", "<=2"),
-                ("1", "<=2"),
-
-                # Test the greater than operation
-                ("3", ">2"),
-                ("2.1", ">2.0"),
-
-                # Test the less than operation
-                ("1", "<2"),
-                ("2.0", "<2.1"),
-            ]
-        ]
-        +
-        [
-            (v, s, False)
-            for v, s in [
-                # Test the equality operation
-                ("2.1", "==2"),
-                ("2.1", "==2.0"),
-                ("2.1", "==2.0.0"),
-
-                # Test the in-equality operation
-                ("2.0", "!=2"),
-                ("2.0", "!=2.0"),
-                ("2.0", "!=2.0.0"),
-
-                # Test the greater than equal operation
-                ("2.0.dev1", ">=2"),
-                ("2.0a1", ">=2"),
-                ("2.0a1.dev1", ">=2"),
-                ("2.0b1", ">=2"),
-                ("2.0b1.post1", ">=2"),
-                ("2.0c1", ">=2"),
-                ("2.0c1.post1.dev1", ">=2"),
-                ("2.0rc1", ">=2"),
-                ("1", ">=2"),
-
-                # Test the less than equal operation
-                ("2.0.post1", "<=2"),
-                ("2.0.post1.dev1", "<=2"),
-                ("3", "<=2"),
-
-                # Test the greater than operation
-                ("1", ">2"),
-                ("2.0.dev1", ">2"),
-                ("2.0a1", ">2"),
-                ("2.0a1.post1", ">2"),
-                ("2.0b1", ">2"),
-                ("2.0b1.dev1", ">2"),
-                ("2.0c1", ">2"),
-                ("2.0c1.post1.dev1", ">2"),
-                ("2.0rc1", ">2"),
-                ("2.0", ">2"),
-
-                # Test the less than operation
-                ("3", "<2"),
-            ]
-        ],
-    )
-    def test_specifiers(self, version, spec, expected):
-        spec = LegacySpecifier(spec, prereleases=True)
-
-        if expected:
-            # Test that the plain string form works
-            assert version in spec
-            assert spec.contains(version)
-
-            # Test that the version instance form works
-            assert LegacyVersion(version) in spec
-            assert spec.contains(LegacyVersion(version))
-        else:
-            # Test that the plain string form works
-            assert version not in spec
-            assert not spec.contains(version)
-
-            # Test that the version instance form works
-            assert LegacyVersion(version) not in spec
-            assert not spec.contains(LegacyVersion(version))
-
-    def test_specifier_explicit_prereleases(self):
-        spec = LegacySpecifier(">=1.0")
-        assert not spec.prereleases
-        spec.prereleases = True
-        assert spec.prereleases
-
-        spec = LegacySpecifier(">=1.0", prereleases=False)
-        assert not spec.prereleases
-        spec.prereleases = True
-        assert spec.prereleases
-
-        spec = LegacySpecifier(">=1.0", prereleases=True)
-        assert spec.prereleases
-        spec.prereleases = False
-        assert not spec.prereleases
-
-        spec = LegacySpecifier(">=1.0", prereleases=True)
-        assert spec.prereleases
-        spec.prereleases = None
-        assert not spec.prereleases
+    def test_specifier_hash_for_compatible_operator(self):
+        assert hash(Specifier("~=1.18.0")) != hash(Specifier("~=1.18"))
 
 
 class TestSpecifierSet:
-
-    @pytest.mark.parametrize(
-        "version",
-        VERSIONS + LEGACY_VERSIONS,
-    )
+    @pytest.mark.parametrize("version", VERSIONS)
     def test_empty_specifier(self, version):
         spec = SpecifierSet(prereleases=True)
 
@@ -868,24 +688,27 @@ class TestSpecifierSet:
         assert spec.contains("1.0.dev1")
         assert not spec.contains("1.0.dev1", prereleases=False)
 
+    def test_specifier_contains_installed_prereleases(self):
+        spec = SpecifierSet("~=1.0")
+        assert not spec.contains("1.0.0.dev1", installed=True)
+        assert spec.contains("1.0.0.dev1", prereleases=True, installed=True)
+
+        spec = SpecifierSet("~=1.0", prereleases=True)
+        assert spec.contains("1.0.0.dev1", installed=True)
+        assert not spec.contains("1.0.0.dev1", prereleases=False, installed=False)
+
     @pytest.mark.parametrize(
-        (
-            "specifier", "specifier_prereleases", "prereleases", "input",
-            "expected",
-        ),
+        ("specifier", "specifier_prereleases", "prereleases", "input", "expected"),
         [
             # General test of the filter method
             ("", None, None, ["1.0", "2.0a1"], ["1.0"]),
             (">=1.0.dev1", None, None, ["1.0", "2.0a1"], ["1.0", "2.0a1"]),
             ("", None, None, ["1.0a1"], ["1.0a1"]),
             ("", None, None, ["1.0", Version("2.0")], ["1.0", Version("2.0")]),
-            ("", None, None, ["2.0dog", "1.0"], ["1.0"]),
-
             # Test overriding with the prereleases parameter on filter
             ("", None, False, ["1.0a1"], []),
             (">=1.0.dev1", None, False, ["1.0", "2.0a1"], ["1.0"]),
             ("", None, True, ["1.0", "2.0a1"], ["1.0", "2.0a1"]),
-
             # Test overriding with the overall specifier
             ("", True, None, ["1.0", "2.0a1"], ["1.0", "2.0a1"]),
             ("", False, None, ["1.0", "2.0a1"], ["1.0"]),
@@ -895,23 +718,18 @@ class TestSpecifierSet:
             ("", False, None, ["1.0a1"], []),
         ],
     )
-    def test_specifier_filter(self, specifier_prereleases, specifier,
-                              prereleases, input, expected):
+    def test_specifier_filter(
+        self, specifier_prereleases, specifier, prereleases, input, expected
+    ):
         if specifier_prereleases is None:
             spec = SpecifierSet(specifier)
         else:
             spec = SpecifierSet(specifier, prereleases=specifier_prereleases)
 
-        kwargs = (
-            {"prereleases": prereleases} if prereleases is not None else {}
-        )
+        kwargs = {"prereleases": prereleases} if prereleases is not None else {}
 
         assert list(spec.filter(input, **kwargs)) == expected
 
-    def test_legacy_specifiers_combined(self):
-        spec = SpecifierSet("<3,>1-1-1")
-        assert "2.0" in spec
-
     @pytest.mark.parametrize(
         ("specifier", "expected"),
         [
@@ -923,10 +741,8 @@ class TestSpecifierSet:
             (">2.0", ">2.0"),
             (">=2.0", ">=2.0"),
             ("~=2.0", "~=2.0"),
-
             # Spaces should be removed
             ("< 2", "<2"),
-
             # Multiple item specifiers should work
             ("!=2.0,>1.0", "!=2.0,>1.0"),
             ("!=2.0 ,>1.0", "!=2.0,>1.0"),
@@ -936,17 +752,14 @@ class TestSpecifierSet:
         spec = SpecifierSet(specifier)
 
         assert str(spec) == expected
-        assert repr(spec) == "<SpecifierSet({0})>".format(repr(expected))
+        assert repr(spec) == f"<SpecifierSet({expected!r})>"
 
     @pytest.mark.parametrize("specifier", SPECIFIERS + LEGACY_SPECIFIERS)
     def test_specifiers_hash(self, specifier):
         assert hash(SpecifierSet(specifier)) == hash(SpecifierSet(specifier))
 
     @pytest.mark.parametrize(
-        ("left", "right", "expected"),
-        [
-            (">2.0", "<5.0", ">2.0,<5.0"),
-        ],
+        ("left", "right", "expected"), [(">2.0", "<5.0", ">2.0,<5.0")]
     )
     def test_specifiers_combine(self, left, right, expected):
         result = SpecifierSet(left) & SpecifierSet(right)
@@ -971,30 +784,26 @@ class TestSpecifierSet:
         assert result == SpecifierSet(expected)
         assert not result.prereleases
 
-        result = (
-            SpecifierSet(left, prereleases=True)
-            & SpecifierSet(right, prereleases=True)
+        result = SpecifierSet(left, prereleases=True) & SpecifierSet(
+            right, prereleases=True
         )
         assert result == SpecifierSet(expected)
         assert result.prereleases
 
-        result = (
-            SpecifierSet(left, prereleases=False)
-            & SpecifierSet(right, prereleases=False)
+        result = SpecifierSet(left, prereleases=False) & SpecifierSet(
+            right, prereleases=False
         )
         assert result == SpecifierSet(expected)
         assert not result.prereleases
 
         with pytest.raises(ValueError):
-            result = (
-                SpecifierSet(left, prereleases=True)
-                & SpecifierSet(right, prereleases=False)
+            result = SpecifierSet(left, prereleases=True) & SpecifierSet(
+                right, prereleases=False
             )
 
         with pytest.raises(ValueError):
-            result = (
-                SpecifierSet(left, prereleases=False)
-                & SpecifierSet(right, prereleases=True)
+            result = SpecifierSet(left, prereleases=False) & SpecifierSet(
+                right, prereleases=True
             )
 
     def test_specifiers_combine_not_implemented(self):
@@ -1006,20 +815,14 @@ class TestSpecifierSet:
         itertools.chain(
             *
             # Verify that the equal (==) operator works correctly
-            [
-                [(x, x, operator.eq) for x in SPECIFIERS]
-            ]
+            [[(x, x, operator.eq) for x in SPECIFIERS]]
             +
             # Verify that the not equal (!=) operator works correctly
             [
-                [
-                    (x, y, operator.ne)
-                    for j, y in enumerate(SPECIFIERS)
-                    if i != j
-                ]
+                [(x, y, operator.ne) for j, y in enumerate(SPECIFIERS) if i != j]
                 for i, x in enumerate(SPECIFIERS)
             ]
-        )
+        ),
     )
     def test_comparison_true(self, left, right, op):
         assert op(SpecifierSet(left), SpecifierSet(right))
@@ -1033,20 +836,14 @@ class TestSpecifierSet:
         itertools.chain(
             *
             # Verify that the equal (==) operator works correctly
-            [
-                [(x, x, operator.ne) for x in SPECIFIERS]
-            ]
+            [[(x, x, operator.ne) for x in SPECIFIERS]]
             +
             # Verify that the not equal (!=) operator works correctly
             [
-                [
-                    (x, y, operator.eq)
-                    for j, y in enumerate(SPECIFIERS)
-                    if i != j
-                ]
+                [(x, y, operator.eq) for j, y in enumerate(SPECIFIERS) if i != j]
                 for i, x in enumerate(SPECIFIERS)
             ]
-        )
+        ),
     )
     def test_comparison_false(self, left, right, op):
         assert not op(SpecifierSet(left), SpecifierSet(right))
@@ -1055,6 +852,31 @@ class TestSpecifierSet:
         assert not op(left, SpecifierSet(right))
         assert not op(SpecifierSet(left), right)
 
+    @pytest.mark.parametrize(("left", "right"), [("==2.8.0", "==2.8")])
+    def test_comparison_canonicalizes(self, left, right):
+        assert SpecifierSet(left) == SpecifierSet(right)
+        assert left == SpecifierSet(right)
+        assert SpecifierSet(left) == right
+
     def test_comparison_non_specifier(self):
         assert SpecifierSet("==1.0") != 12
         assert not SpecifierSet("==1.0") == 12
+
+    @pytest.mark.parametrize(
+        ("version", "specifier", "expected"),
+        [
+            ("1.0.0+local", "==1.0.0", True),
+            ("1.0.0+local", "!=1.0.0", False),
+            ("1.0.0+local", "<=1.0.0", True),
+            ("1.0.0+local", ">=1.0.0", True),
+            ("1.0.0+local", "<1.0.0", False),
+            ("1.0.0+local", ">1.0.0", False),
+        ],
+    )
+    def test_comparison_ignores_local(self, version, specifier, expected):
+        assert (Version(version) in SpecifierSet(specifier)) == expected
+
+    def test_contains_with_compatible_operator(self):
+        combination = SpecifierSet("~=1.18.0") & SpecifierSet("~=1.18")
+        assert "1.19.5" not in combination
+        assert "1.18.0" in combination
